#!/usr/bin/python3

from pympa import *
import time
import sys
import argparse
import atexit
import copy
import numpy as np
import cv2

parser = argparse.ArgumentParser(description='pympa-focus-helper')
parser.add_argument('-i', '--input',      type=str,   required=False, default='hires_color')
parser.add_argument('-o', '--output',     type=str,   required=False, default='hires_debug')
parser.add_argument('-r', '--rate',       type=float, required=False, default=30.0)
parser.add_argument('-z', '--zoom',       type=float, required=False, default=12.0)
parser.add_argument('-b', '--box-size',   type=int,   required=False, default=100.0)
parser.add_argument('-s', '--size',       type=str,   required=False, default="1920x1080")
args = parser.parse_args()

app_name    = 'pympa-focus-helper'

input_name  = args.input
output_name = args.output
pub_rate    = args.rate
dig_zoom    = args.zoom


#if input size is smaller, the output size will be set to input size
out_dims    = args.size.split('x')
output_size = [int(out_dims[0]),int(out_dims[1])]
print('output image dimensions : ', output_size)

roi_size     = np.array((args.box_size,args.box_size),dtype='uint32')
roi_shape_up = (roi_size*dig_zoom).astype('uint32')  #upscaled (zoomed) roi size

def exit_handler():
    pympa.cleanup()

atexit.register(exit_handler)

#pre-allocate array for input image
#TODO: image size can be queried before getting the image
max_width  = 5000;
max_height = 4000;
image_width  = np.zeros(1,dtype='uint32')
image_height = np.zeros(1,dtype='uint32')
image_buffer = np.zeros(max_width*max_height, dtype='uint8')

pympa_subscribe(input_name, app_name)   #subscribe to input image
pympa_create_pub(output_name, app_name) #create output image publisher

t_next = time.time() + 1.0 / pub_rate

frame_counter = 0

while True:

    # update rate controller
    t_now = time.time()
    dt    = t_next - t_now
    if dt > 0:
        time.sleep(dt)
    t_next = t_next + 1.0 / pub_rate
    
    meta     = camera_image_metadata_t()
    img_size = pympa_get_image(image_buffer, meta)
    
    if img_size > 0:
        frame_counter += 1
        print('.',end='')   #print progress
        sys.stdout.flush()
        
        image_width  = meta.width
        image_height = meta.height
        if frame_counter == 1:
            print('width=%d, height=%d, format=%d, size=%d' % (meta.width, meta.height, meta.format, meta.size_bytes))
        
        if output_size[0] > image_width or output_size[1] > image_height:
            output_size = [image_width, image_height]
            
        if roi_shape_up[0] > image_width/2 or roi_shape_up[1] > image_height/2:
            roi_size     = np.array((image_width/3/dig_zoom,image_width/3/dig_zoom),dtype='uint32')
            roi_shape_up = (roi_size*dig_zoom).astype('uint32')  #upscaled (zoomed) roi size
        
        img = image_buffer[0:int(image_width*image_height)]
        img = np.reshape(img, (int(image_height),int(image_width)))
               
        #extract the ROI from the original image
        start_point = (int(image_width/2-roi_size[0]/2),int(image_height/2-roi_size[1]/2))
        end_point   = (int(image_width/2+roi_size[0]/2),int(image_height/2+roi_size[1]/2))
        img_roi     = img[start_point[1]:end_point[1], start_point[0]:end_point[0]];
                
        #edge detection on ROI
        img_edge = cv2.Canny(img_roi,100,105,7)
        
        #upscale the ROI and edge detector output
        img_roi_up   = cv2.resize(img_roi,  roi_shape_up)
        img_edge_up  = cv2.resize(img_edge, roi_shape_up)
        
        #overwrite the upscaled images onto the original image
        img[0:roi_shape_up[1],0:roi_shape_up[0]] = img_roi_up
        img[0:roi_shape_up[1],(image_width-roi_shape_up[0]):image_width] = img_edge_up
        
        #draw a bounding box for roi
        color = (255, 0, 0) 
        thickness = 2
        cv2.rectangle(img, start_point, end_point, color, thickness)
        
        #resize to the desired output resolution
        out_image           = cv2.resize(img, output_size)
        
        #create metadata structure for output image - re-use the original meta
        meta_out            = copy.deepcopy(meta)
        meta_out.width      = out_image.shape[1]
        meta_out.height     = out_image.shape[0]
        meta_out.size_bytes = meta_out.width * meta_out.height 
        meta_out.format     = 0 # 0:IMAGE_FORMAT_RAW8, 1:IMAGE_FORMAT_NV12, 10:IMAGE_FORMAT_RGB
        
        pympa_publish_image(out_image,meta_out)
        
    else:
        print('waiting for the first image')

